#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.core import exceptions
from django.db import models
from django.utils.translation import ugettext_lazy as _


class FlowCharField(models.CharField):
    "Custom CharField with workflow validation"

    @staticmethod
    def get_db_value(model_instance):
        "Get database value"
        if model_instance.id:
            return model_instance.__class__.objects.get(id=model_instance.id)

    def validate(self, value, model_instance):
        "Validate choice workflow"
        if model_instance.id:
            db_value = self.get_db_value(model_instance)
            if not self.choices.validate(getattr(db_value, self.name), value):
                raise exceptions.ValidationError(_('%s %s' % (self.choices.error_msg,
                                                 self.choices.get_value(value))))

        super(FlowCharField, self).validate(value, model_instance)


class FlowIntegerField(models.IntegerField):
    "Custom IntegerField with workflow validation"

    @staticmethod
    def get_db_value(model_instance):
        "Get database value"
        if model_instance.id:
            return model_instance.__class__.objects.get(id=model_instance.id)

    def validate(self, value, model_instance):
        "Validate choice workflow"
        if model_instance.id:
            db_value = self.get_db_value(model_instance)
            if not self.choices.validate(getattr(db_value, self.name), value):
                raise exceptions.ValidationError(_('%s %s' % (self.choices.error_msg,
                                                 self.choices.get_value(value))))

        super(FlowIntegerField, self).validate(value, model_instance)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^django_choices_flow\.models\.(FlowCharField|FlowIntegerField)"])
except ImportError:
    pass



# -*- coding: utf-8 -*-
try:
    from django.conf import settings
except ImportError:
    settings = None

if hasattr(settings, 'DJANGO_CHOICES_FLOW_ERROR_MESSAGE'):
    DEFAULT_DJANGO_CHOICES_FLOW_ERROR_MESSAGE = settings.DJANGO_CHOICES_FLOW_ERROR_MESSAGE
else:
    DEFAULT_DJANGO_CHOICES_FLOW_ERROR_MESSAGE = 'Invalid choice:'


class MetaChoice(type):
    """
        - Convert attributes to Tuples
        - Add interator in class attributes
    """
    def __init__(cls, *args, **kwargs):
        cls._rules = {}
        cls._data = []
        cls.__validate_msg = DEFAULT_DJANGO_CHOICES_FLOW_ERROR_MESSAGE

        items = cls.__dict__.items()
        for name, value in items:
            cls._set_data(name, value)
        cls._hash = dict(cls._data)

    def _set_data(cls, name, value, parent_name=None):
        "Update tuplas to objects"
        if not name.startswith('_') and not callable(value):
            if isinstance(value, tuple) and len(value) > 1:
                data = value
            else:
                data = (value, name)

            if isinstance(value, list) and '_RULES' in name.upper():
                cls._set_rules(name, value)
            else:
                cls._data.append(data)
                if isinstance(data[0], tuple):
                    setattr(cls, name, dict(data))
                else:
                    setattr(cls, name, data[0])

    def _set_rules(cls, name, value):
        "If attribute has _RULES add validations rules"
        updated_name = name.rsplit('_', 1)[0]
        value_key = getattr(cls, updated_name)

        if isinstance(value_key, tuple):
            value_key = value_key[0]

        values = [isinstance(i, tuple) and i[0] or i for i in value]
        cls._rules.update({value_key: values})

    def __iter__(cls, *args, **kwds):
        for value, data in cls._data:
            yield (value, data)

    def __len__(cls, *args, **kwds):
        return len(cls._data)

    def __repr__(cls, *args, **kargs):
        return str(list(cls._data))

    @property
    def error_msg(cls):
        "get default error message"
        return cls.__validate_msg

    @error_msg.setter
    def error_msg(cls, value):
        "set default error message for instance"
        cls.__validate_msg = value

    def get_value(cls, key):
        return cls._hash.get(key)

    def validate(cls, status, new_status):
        " Validate workflow "
        if not status:
            return new_status

        if repr(status) == repr(new_status):
            return new_status

        if not cls._rules.get(status) or \
            new_status not in cls._rules.get(status):
            return False

        return new_status

#: Set metaclass in Python 2.x and 3.x
Choices = MetaChoice('Choices', (object, ), {})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

import django_choices_flow

install_requires = [
    'django>=1.2',
]

files = [
    "./django_choices_flow/__init__.py",
    "./django_choices_flow/base.py",
    "./django_choices_flow/models.py",
]

setup(name='django_choices_flow',
      url='https://github.com/valdergallo/django-choices-flow',
      author="valdergallo",
      author_email='valdergallo@gmail.com',
      keywords=['django', 'choices', 'field', 'flow', 'choicesfield', 'control'],
      description='Simple library with flow in choices values for Django',
      license='FREEBSD',
      long_description=('''Django Choices Flow simple library to control flow with choices in Django Model'''),
      classifiers=[
          'Framework :: Django',
          'Operating System :: OS Independent',
          'Topic :: Utilities',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.3',
      ],
      include_package_data=True,
      version=django_choices_flow.__version__,
      install_requires=install_requires,
      packages=['django_choices_flow'],
      package_data={'package': files},
)
